@@ -8,14 +8,14 @@ from tabml.feature_manager import BaseFeatureManager, BaseTransformingFeature
 
 
 class FeatureManager(BaseFeatureManager):
-    def __init__(self, pb_config_path, transformer_path=None):
-        super().__init__(pb_config_path, transformer_path=transformer_path)
+    def __init__(self, feature_config_path, transformer_path=None):
+        super().__init__(feature_config_path, transformer_path=transformer_path)
 
     def _get_base_transforming_class(self):
         return BaseTitanicTransformingFeature
 
     def load_raw_data(self):
-        df_dict = datasets.download_titanic()
+        df_dict = datasets.load_titanic(data_dir=self.raw_data_dir)
         full_df = pd.concat(
             [df_dict["train"], df_dict["test"]], axis=0, ignore_index=True
         )
@@ -173,8 +173,8 @@ class FeatureMinMaxScaledAge(BaseTitanicTransformingFeature):
 
 
 def run(transformer_path=None):
-    pb_config_path = "configs/feature_config.yaml"
-    fm = FeatureManager(pb_config_path, transformer_path)
+    feature_config_path = "configs/feature_config.yaml"
+    fm = FeatureManager(feature_config_path, transformer_path)
     fm.run_all()
 
 
