@@ -20,16 +20,17 @@ class ModelInference:
         self.fm = feature_manager_cls(
             feature_config_path, transformer_path=transformer_path
         )
+        self.fm.load_transformers()
         if pipeline_config_path is None:
             pipeline_config_path = ExperimentManger.get_config_path_from_model_path(
                 model_path
             )
         self.config = parse_pipeline_config(pipeline_config_path)
         if custom_model_wrapper:
-            self.model_wrapper = custom_model_wrapper(self.config)
+            self.model_wrapper = custom_model_wrapper(self.config.model_wrapper)
         else:
             self.model_wrapper = factory.create(self.config.model_wrapper.cls_name)(
-                self.config
+                self.config.model_wrapper
             )
         self.model_wrapper.load_model(model_path)
         self.features_to_model = list(self.config.data_loader.features_to_model)
