@@ -1,4 +1,5 @@
 from abc import ABC
+from pathlib import Path
 
 import mlflow
 
@@ -32,7 +33,7 @@ class BasePipeline(ABC):
         logger.info(f"Running pipeline with config {path_to_config}")
         logger.info("=" * 80)
         self.exp_manager = experiment_manager.ExperimentManger(
-            path_to_config, custom_run_dir=custom_run_dir
+            path_to_config, custom_run_dir=Path(custom_run_dir)
         )
         self.config = parse_pipeline_config(path_to_config)
         self.data_loader = self._get_data_loader()
@@ -48,7 +49,12 @@ class BasePipeline(ABC):
         model_wrappers.MLFLOW_AUTOLOG[model_type]
         with mlflow.start_run():
             mlflow.log_param("model_type", model_type)
-            mlflow.log_params(self.model_wrapper.params)
+            mlflow.log_params(
+                {
+                    "model_params": self.config.model_wrapper.model_params,
+                    "fit_params": self.config.model_wrapper.fit_params,
+                }
+            )
             self.train()
             self.analyze_model()
 
@@ -58,14 +64,14 @@ class BasePipeline(ABC):
         self.model_wrapper.fit(self.data_loader, model_dir)
 
     def _get_data_loader(self) -> BaseDataLoader:
-        return factory.create(self.config.data_loader.cls_name)(self.config)
+        return factory.create(self.config.data_loader.cls_name)(self.config.data_loader)
 
     def _get_model_wrapper(self, custom_model_wrapper):
         if custom_model_wrapper:
-            self.model_wrapper = custom_model_wrapper(self.config)
+            self.model_wrapper = custom_model_wrapper(self.config.model_wrapper)
         else:
             self.model_wrapper = factory.create(self.config.model_wrapper.cls_name)(
-                self.config
+                self.config.model_wrapper
             )
 
     def analyze_model(self) -> None:
@@ -87,9 +93,6 @@ class BasePipeline(ABC):
         ModelAnalysis(
             data_loader=self.data_loader,
             model_wrapper=self.model_wrapper,
-            features_to_analyze=self.config.model_analysis.by_features,
-            label_to_analyze=self.config.model_analysis.by_label,
-            metric_names=self.config.model_analysis.metrics,
+            params=self.config.model_analysis,
             output_dir=self.exp_manager.get_model_analysis_dir(),
-            training_size=self.config.model_analysis.training_size,
         ).analyze()
